@@ -1,6 +1,8 @@
 import os
+import logging
 from importlib import import_module
 
+from .argo_objects import ArgoStep, ArgoWorkflow
 from .common import LineageClient, LocalArtifact, S3Artifact
 from .config import config, s3_config, set_config, set_s3_config
 from .context import Context
@@ -13,14 +15,18 @@ from .op_template import (OPTemplate, PythonScriptOPTemplate, Secret,
                           ShellOPTemplate)
 from .resource import Resource
 from .slurm import SlurmJob, SlurmJobTemplate, SlurmRemoteExecutor
-from .step import (Step, argo_concat, argo_len, argo_range, argo_sequence,
-                   argo_sum)
+from .step import (Step, argo_concat, argo_enumerate, argo_len, argo_range,
+                   argo_sequence, argo_sum)
 from .steps import Steps
 from .task import Task
 from .utils import (copy_artifact, copy_s3, download_artifact, download_s3,
                     path_list_of_artifact, randstr, upload_artifact, upload_s3)
-from .workflow import (DockerSecret, Workflow, query_archived_workflows,
-                       query_workflows)
+from .workflow import (DockerSecret, Workflow, parse_repo,
+                       query_archived_workflows, query_workflows)
+
+log_level = os.environ.get('LOG_LEVEL')
+if log_level:
+    logging.basicConfig(level=getattr(logging, log_level.upper(), None))
 
 __all__ = ["S3Artifact", "DAG", "Executor", "RemoteExecutor", "AutonamedDict",
            "IfExpression", "InputArtifact", "InputParameter", "Inputs",
@@ -34,7 +40,8 @@ __all__ = ["S3Artifact", "DAG", "Executor", "RemoteExecutor", "AutonamedDict",
            "Context", "randstr", "LocalArtifact", "set_config",
            "set_s3_config", "DockerSecret", "argo_sum", "argo_concat",
            "LineageClient", "Secret", "query_workflows",
-           "query_archived_workflows", "ContainerExecutor"]
+           "query_archived_workflows", "ContainerExecutor", "ArgoStep",
+           "ArgoWorkflow", "argo_enumerate"]
 
 
 def import_func(s):
@@ -52,3 +59,4 @@ if os.environ.get("DFLOW_LINEAGE"):
 if os.environ.get("DFLOW_S3_STORAGE_CLIENT"):
     s3_config["storage_client"] = import_func(os.environ.get(
         "DFLOW_S3_STORAGE_CLIENT"))()
+parse_repo()
