@@ -7,7 +7,7 @@ from typing import Any, Dict, List, Optional, Union
 
 import jsonpickle
 
-from .common import S3Artifact
+from .common import CustomArtifact, S3Artifact
 from .config import config
 from .utils import randstr, s3_config, upload_s3
 
@@ -266,12 +266,35 @@ class ArgoVar:
         return ArgoVar("asFloat(%s) >= %s" % (self.expr, other))
 
     def __add__(self, other):
+        if isinstance(other, str):
+            return ArgoVar("%s + '%s'" % (self.expr, other))
         if config["mode"] == "debug":
             return ArgoVar("%s + %s" % (self.expr, to_expr(other)))
         if isinstance(other, ArgoVar):
             other = "asFloat(%s)" % other.expr
         return ArgoVar("asFloat(%s) + %s" % (self.expr, other))
 
+    def __sub__(self, other):
+        if config["mode"] == "debug":
+            return ArgoVar("%s - %s" % (self.expr, to_expr(other)))
+        if isinstance(other, ArgoVar):
+            other = "asFloat(%s)" % other.expr
+        return ArgoVar("asFloat(%s) - %s" % (self.expr, other))
+
+    def __mul__(self, other):
+        if config["mode"] == "debug":
+            return ArgoVar("%s * %s" % (self.expr, to_expr(other)))
+        if isinstance(other, ArgoVar):
+            other = "asFloat(%s)" % other.expr
+        return ArgoVar("asFloat(%s) * %s" % (self.expr, other))
+
+    def __truediv__(self, other):
+        if config["mode"] == "debug":
+            return ArgoVar("%s / %s" % (self.expr, to_expr(other)))
+        if isinstance(other, ArgoVar):
+            other = "asFloat(%s)" % other.expr
+        return ArgoVar("asFloat(%s) / %s" % (self.expr, other))
+
 
 class IfExpression(ArgoVar, Expression):
     def __init__(
@@ -347,7 +370,11 @@ def convert_value_to_str(value):
                     else enumerate(obj):
                 if isinstance(v, ArgoVar):
                     obj[i] = "dflow_placeholder_%s" % len(vars)
-                    vars.append("{{=%s}}" % v.expr)
+                    if (hasattr(v, "type") and v.type == str) or (
+                            hasattr(v, "value") and isinstance(v.value, str)):
+                        vars.append("\"{{=%s}}\"" % v.expr)
+                    else:
+                        vars.append("{{=%s}}" % v.expr)
                 elif isinstance(v, (dict, list)):
                     handle(v)
 
@@ -532,6 +559,7 @@ class InputArtifact(ArgoVar):
             mode: Optional[int] = None,
             sub_path: Optional[str] = None,
             archive: str = "default",
+            save_as_parameter: bool = False,
             **kwargs,
     ) -> None:
         self.path = path
@@ -543,10 +571,11 @@ class InputArtifact(ArgoVar):
         self.source = source
         self._sub_path = None
         self.mode = mode
-        self.sub_path = sub_path
+        self.sp = sub_path
         self.archive = archive
         self.slice = None
         self.parent = None
+        self.save_as_parameter = save_as_parameter
         for k, v in kwargs.items():
             self.__setattr__(k, v)
 
@@ -592,6 +621,17 @@ class InputArtifact(ArgoVar):
 
     def __repr__(self):
         from .task import Task
+        if self.save_as_parameter:
+            if self.name is not None:
+                if self.step is not None:
+                    if isinstance(self.step, Task):
+                        return "{{tasks.%s.inputs.parameters.dflow_art_%s}}"\
+                            % (self.step.id, self.name)
+                    else:
+                        return "{{steps.%s.inputs.artifacts.dflow_art_%s}}"\
+                            % (self.step.id, self.name)
+                return "{{inputs.artifacts.dflow_art_%s}}" % self.name
+            return ""
         if self.name is not None:
             if self.step is not None:
                 if isinstance(self.step, Task):
@@ -605,7 +645,10 @@ class InputArtifact(ArgoVar):
 
     def sub_path(self, path):
         artifact = deepcopy(self)
-        artifact._sub_path = path
+        if artifact._sub_path is None:
+            artifact._sub_path = path
+        else:
+            artifact._sub_path += "/" + path
         return artifact
 
     def get_path_list_parameter(self):
@@ -616,6 +659,16 @@ class InputArtifact(ArgoVar):
         return self.template.inputs.parameters["dflow_%s_urn" % self.name]
 
     def convert_to_argo(self):
+        if self.save_as_parameter:
+            if self.source is None:
+                return V1alpha1Parameter(name="dflow_art_%s" % self.name)
+            elif isinstance(self.source, CustomArtifact):
+                return V1alpha1Parameter(name="dflow_art_%s" % self.name,
+                                         value=jsonpickle.dumps(self.source))
+            else:
+                return V1alpha1Parameter(name="dflow_art_%s" % self.name,
+                                         value=str(self.source))
+
         archive = None
         if self.archive is None:
             archive = V1alpha1ArchiveStrategy(_none={})
@@ -627,8 +680,10 @@ class InputArtifact(ArgoVar):
                                     optional=self.optional, mode=self.mode,
                                     archive=archive)
         if isinstance(self.source, (InputArtifact, OutputArtifact)):
-            sub_path = self.sub_path if self.sub_path is not None else \
+            sub_path = self.sp if self.sp is not None else \
                 self.source._sub_path
+            if isinstance(sub_path, ArgoVar):
+                sub_path = "{{=%s}}" % sub_path.expr
             return V1alpha1Artifact(name=self.name, path=self.path,
                                     optional=self.optional,
                                     _from=str(self.source), sub_path=sub_path,
@@ -637,13 +692,13 @@ class InputArtifact(ArgoVar):
             if s3_config["repo_type"] == "s3":
                 return V1alpha1Artifact(name=self.name, path=self.path,
                                         optional=self.optional, s3=self.source,
-                                        sub_path=self.sub_path, mode=self.mode,
+                                        sub_path=self.sp, mode=self.mode,
                                         archive=archive)
             else:
                 return V1alpha1Artifact(name=self.name, path=self.path,
                                         optional=self.optional,
                                         oss=self.source.oss(),
-                                        sub_path=self.sub_path, mode=self.mode,
+                                        sub_path=self.sp, mode=self.mode,
                                         archive=archive)
         elif isinstance(self.source, str):
             return V1alpha1Artifact(name=self.name, path=self.path,
@@ -974,7 +1029,10 @@ class OutputArtifact(ArgoVar):
 
     def sub_path(self, path):
         artifact = deepcopy(self)
-        artifact._sub_path = path
+        if artifact._sub_path is None:
+            artifact._sub_path = path
+        else:
+            artifact._sub_path += "/" + path
         return artifact
 
     def __getattr__(self, key):
@@ -1178,7 +1236,10 @@ class Inputs:
             else:
                 parameters.append(par.convert_to_argo())
         for art in self.artifacts.values():
-            artifacts.append(art.convert_to_argo())
+            if art.save_as_parameter:
+                parameters.append(art.convert_to_argo())
+            else:
+                artifacts.append(art.convert_to_argo())
         return V1alpha1Inputs(parameters=parameters, artifacts=artifacts)
 
 
