@@ -1,13 +1,17 @@
+import abc
 import json
 import os
+from abc import ABC
 from copy import deepcopy
+from getpass import getpass
 from typing import List, Optional, Union
 
-from ..common import S3Artifact
+from ..common import CustomArtifact, S3Artifact
 from ..config import config
 from ..executor import Executor, render_script_with_tmp_root, run_script
 from ..io import InputArtifact, InputParameter
 from ..op_template import ScriptOPTemplate
+from ..python import PythonOPTemplate
 from ..utils import randstr, upload_s3
 from . import bohrium
 
@@ -76,6 +80,9 @@ class DispatcherExecutor(Executor):
                  remote_root: Optional[str] = None,
                  retry_on_submission_error: Optional[int] = None,
                  merge_sliced_step: bool = False,
+                 pre_script: str = "",
+                 post_script: str = "",
+                 clean: bool = True,
                  ) -> None:
         self.host = host
         self.queue_name = queue_name
@@ -110,6 +117,9 @@ class DispatcherExecutor(Executor):
         self.remote_root = remote_root
         self.retry_on_submission_error = retry_on_submission_error
         self.merge_sliced_step = merge_sliced_step
+        self.pre_script = pre_script
+        self.post_script = post_script
+        self.clean = clean
 
         conf = {}
         if json_file is not None:
@@ -132,8 +142,11 @@ class DispatcherExecutor(Executor):
         if self.remote_root is not None:
             self.machine_dict["remote_root"] = self.remote_root
         else:
-            self.machine_dict["remote_root"] = "/home/%s/dflow/workflows" % \
-                self.username
+            if self.username == "root":
+                self.machine_dict["remote_root"] = "/root/dflow/workflows"
+            else:
+                self.machine_dict["remote_root"] = "/home/%s/dflow/workflows"\
+                    % self.username
         if self.private_key_file is not None:
             self.machine_dict["remote_profile"]["key_filename"] = \
                 "/root/.ssh/" + os.path.basename(self.private_key_file)
@@ -145,18 +158,27 @@ class DispatcherExecutor(Executor):
         if self.machine_dict["context_type"] == "Bohrium":
             if "batch_type" not in self.machine_dict:
                 self.machine_dict["batch_type"] = "Bohrium"
-            if "email" not in self.machine_dict["remote_profile"] and \
-                    bohrium.config["username"] is not None:
-                self.machine_dict["remote_profile"]["email"] = \
-                    bohrium.config["username"]
-            if "password" not in self.machine_dict["remote_profile"] and \
-                    bohrium.config["password"] is not None:
-                self.machine_dict["remote_profile"]["password"] = \
-                    bohrium.config["password"]
-            if "program_id" not in self.machine_dict["remote_profile"] and \
-                    bohrium.config["project_id"] is not None:
-                self.machine_dict["remote_profile"]["program_id"] = int(
-                    bohrium.config["project_id"])
+            if "email" not in self.machine_dict["remote_profile"]:
+                if bohrium.config["username"] is not None:
+                    self.machine_dict["remote_profile"]["email"] = \
+                        bohrium.config["username"]
+                else:
+                    self.machine_dict["remote_profile"]["email"] = input(
+                        "Bohrium email: ")
+            if "password" not in self.machine_dict["remote_profile"]:
+                if bohrium.config["password"] is not None:
+                    self.machine_dict["remote_profile"]["password"] = \
+                        bohrium.config["password"]
+                else:
+                    self.machine_dict["remote_profile"]["password"] = getpass(
+                        "Bohrium password: ")
+            if "program_id" not in self.machine_dict["remote_profile"]:
+                if bohrium.config["project_id"] is not None:
+                    self.machine_dict["remote_profile"]["program_id"] = int(
+                        bohrium.config["project_id"])
+                else:
+                    self.machine_dict["remote_profile"]["program_id"] = int(
+                        input("Bohrium program ID: "))
             if "input_data" not in self.machine_dict["remote_profile"]:
                 self.machine_dict["remote_profile"]["input_data"] = {}
             input_data = self.machine_dict["remote_profile"]["input_data"]
@@ -242,8 +264,25 @@ class DispatcherExecutor(Executor):
                 self.task_dict["backward_files"].append(
                     "./" + par.value_from_path)
 
-        new_template.script = "import os\n"
+        new_template.script = self.pre_script
+        new_template.script += "import os\n"
         new_template.script += "os.chdir('%s')\n" % self.work_root
+        if any([art.save_as_parameter for art in
+                template.inputs.artifacts.values()]) and \
+                isinstance(template, PythonOPTemplate):
+            template = deepcopy(template)
+            if self.machine_dict["context_type"] == "Bohrium":
+                template.download_method = "bohrium_download"
+            elif self.machine_dict["context_type"] == "SSHContext":
+                template.download_method = "remote_download"
+            template.render_script()
+            new_template.volumes = [v for v in new_template.volumes
+                                    if v.name not in ["launching", "dev-fuse"]]
+            new_template.mounts = [m for m in new_template.mounts
+                                   if m.name != "launching"]
+            new_template.sidecars = [s for s in new_template.sidecars
+                                     if not s.name.startswith("rclone-")]
+
         new_template.script += "with open('script', 'w') as f:\n"
         new_template.script += "    f.write(r\"\"\"\n"
         if self.map_tmp_dir:
@@ -267,6 +306,13 @@ class DispatcherExecutor(Executor):
             " Task, Submission\n"
         new_template.script += "machine = Machine.load_from_dict(json.loads("\
             "r'%s'))\n" % json.dumps(self.machine_dict)
+        for name, art in template.inputs.artifacts.items():
+            if self.machine_dict["context_type"] == "Bohrium" and \
+                    art.save_as_parameter:
+                new_template.script += "machine.input_data['job_resources']"\
+                    " = machine.input_data.get('job_resources', []) + ['%s']"\
+                    "\n" % art.source.get_bohrium_urn(name)
+
         new_template.script += "resources = Resources.load_from_dict(json."\
             "loads(r'%s'))\n" % json.dumps(self.resources_dict)
         if new_template.envs is not None:
@@ -370,7 +416,8 @@ class DispatcherExecutor(Executor):
                 self.retry_on_submission_error
             new_template.script += "    try:\n"
             new_template.script += "        print('retry ' + str(retry))\n"
-            new_template.script += "        submission.run_submission()\n"
+            new_template.script += "        submission.run_submission(clean="\
+                "%s)\n" % self.clean
             new_template.script += "        break\n"
             new_template.script += "    except Exception:\n"
             new_template.script += "        import traceback\n"
@@ -378,7 +425,8 @@ class DispatcherExecutor(Executor):
             new_template.script += "        import time\n"
             new_template.script += "        time.sleep(2**retry)\n"
         else:
-            new_template.script += "submission.run_submission()\n"
+            new_template.script += "submission.run_submission(clean=%s)\n" % \
+                self.clean
         if merge:
             for name in sliced_output_parameters:
                 path = template.outputs.parameters[name].value_from_path
@@ -402,6 +450,7 @@ class DispatcherExecutor(Executor):
                 new_template.script += "assert os.path.exists('./%s')\n" % \
                     par.value_from_path
 
+        new_template.script += self.post_script
         if self.private_key_file is not None:
             key = upload_s3(self.private_key_file)
             private_key_artifact = S3Artifact(key=key)
@@ -415,3 +464,17 @@ class DispatcherExecutor(Executor):
             new_template.mounts.append(V1VolumeMount(
                 name="dflow-private-key", mount_path="/root/.ssh"))
         return new_template
+
+
+class DispatcherArtifact(CustomArtifact, ABC):
+    @abc.abstractmethod
+    def get_bohrium_urn(self, name: str) -> str:
+        pass
+
+    @abc.abstractmethod
+    def remote_download(self, name: str, path: str):
+        pass
+
+    @abc.abstractmethod
+    def bohrium_download(self, name: str, path: str):
+        pass
