@@ -1,7 +1,9 @@
 import inspect
+import logging
 import os
+import sys
 from pathlib import Path
-from typing import Optional, Any, Dict, List, Union
+from typing import Any, Dict, List, Optional, Type, Union
 
 import jsonpickle
 import typeguard
@@ -12,8 +14,8 @@ from ..config import config
 from ..io import (PVC, InputArtifact, InputParameter, Inputs, OutputArtifact,
                   OutputParameter, Outputs)
 from ..op_template import PythonScriptOPTemplate
-from ..utils import s3_config, randstr
-from .op import OP
+from ..utils import add_prefix_to_slice, randstr, s3_config
+from .op import OP, iwd
 from .opio import Artifact, BigParameter, Parameter
 
 try:
@@ -54,6 +56,8 @@ class Slices:
             output_artifact: Optional[List[str]] = None,
             sub_path: bool = False,
             group_size: Optional[int] = None,
+            shuffle: bool = False,
+            random_seed: int = 0,
             pool_size: Optional[int] = None,
             register_first_only: bool = False,
     ) -> None:
@@ -73,18 +77,38 @@ class Slices:
         else:
             self.slices = "{{item}}"
         self.group_size = group_size
+        self.shuffle = shuffle
+        self.random_seed = random_seed
         self.pool_size = pool_size
         self.register_first_only = register_first_only
 
 
 def get_source_code(o):
     source_lines, start_line = inspect.getsourcelines(o)
-    with open(inspect.getsourcefile(o), "r",
-              encoding="utf-8") as fd:
+    if sys.version_info.minor >= 9:
+        source_file = inspect.getsourcefile(o)
+    else:
+        source_file = os.path.join(iwd, inspect.getsourcefile(o))
+    with open(source_file, "r", encoding="utf-8") as fd:
         pre_lines = fd.readlines()[:start_line-1]
     return "".join(pre_lines + source_lines) + "\n"
 
 
+def handle_packages_script(package_root):
+    script = "import os, sys, json\n"
+    script += "package_root = r'%s'\n" % package_root
+    script += "catalog_dir = os.path.join(package_root, "\
+        "'%s')\n" % config['catalog_dir_name']
+    script += "if os.path.exists(catalog_dir):\n"
+    script += "    for f in os.listdir(catalog_dir):\n"
+    script += "        with open(os.path.join(catalog_dir, f), 'r')"\
+        " as fd:\n"
+    script += "            for item in json.load(fd)['path_list']:\n"
+    script += "                sys.path.insert(0, os.path.join("\
+        "package_root, os.path.dirname(item['dflow_list_item'])))\n"
+    return script
+
+
 class PythonOPTemplate(PythonScriptOPTemplate):
     """
     Convert from Python class OP to OP template
@@ -124,16 +148,17 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         limits: a dict of resource limits
         envs: environment variables
         init_containers: init containers before the template runs
+        sidecars: sidecar containers
     """
 
     def __init__(self,
-                 op_class: OP,
+                 op_class: Union[Type[OP], OP],
                  image: Optional[str] = None,
                  command: Union[str, List[str]] = None,
                  output_artifact_save: Dict[str,
                                             List[Union[PVC, S3Artifact]]]
                  = None,
-                 output_artifact_archive: Dict[str, str] = None,
+                 output_artifact_archive: Dict[str, Optional[str]] = None,
                  output_parameter_default: Dict[str, Any] = None,
                  input_artifact_slices: Dict[str, str] = None,
                  input_parameter_slices: Dict[str, str] = None,
@@ -155,6 +180,7 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                  upload_dflow: bool = True,
                  envs: Dict[str, str] = None,
                  init_containers: Optional[List[V1alpha1UserContainer]] = None,
+                 sidecars: Optional[List[V1alpha1UserContainer]] = None,
                  tmp_root: str = "/tmp",
                  pre_script: str = "",
                  post_script: str = "",
@@ -182,18 +208,13 @@ class PythonOPTemplate(PythonScriptOPTemplate):
             name="%s-%s" % (class_name, randstr()), inputs=Inputs(),
             outputs=Outputs(), volumes=volumes, mounts=mounts,
             requests=requests, limits=limits, envs=envs,
-            init_containers=init_containers)
+            init_containers=init_containers, sidecars=sidecars)
         self.pre_script = pre_script
         self.post_script = post_script
         if timeout is not None:
             self.timeout = "%ss" % timeout
-        if retry_on_transient_error is not None:
-            if timeout_as_transient_error:
-                expr = "asInt(lastRetry.exitCode) != 2"
-            else:
-                expr = "asInt(lastRetry.exitCode) == 1"
-            self.retry_strategy = V1alpha1RetryStrategy(
-                limit=retry_on_transient_error, expression=expr)
+        self.retry_on_transient_error = retry_on_transient_error
+        self.timeout_as_transient_error = timeout_as_transient_error
         self.dflow_vars = {}
         self.tmp_root = tmp_root
         for name, sign in input_sign.items():
@@ -316,48 +337,52 @@ class PythonOPTemplate(PythonScriptOPTemplate):
             else output_artifact_slices
         self.output_parameter_slices = {} if output_parameter_slices is None \
             else output_parameter_slices
+        self.set_slices(slices)
+        self.download_method = "download"
+
+    def set_slices(self, slices, input_artifact_prefix=None):
         self.slices = slices
-
-    def __setattr__(self, key, value):
-        super().__setattr__(key, value)
-        if key == "slices":
-            self.init_slices(value)
-            self.render_script()
-
-    def init_slices(self, slices):
         self.input_artifact_slices = {}
         self.input_parameter_slices = {}
         self.output_artifact_slices = {}
         self.output_parameter_slices = {}
-        self.add_slices(slices)
-
-    def add_slices(self, slices):
         if slices is not None:
-            assert isinstance(slices, Slices)
-            if slices.input_artifact and not slices.sub_path:
-                for name in slices.input_artifact:
-                    self.input_artifact_slices[name] = slices.slices
-            if slices.input_parameter:
-                for name in slices.input_parameter:
-                    self.input_parameter_slices[name] = slices.slices
-            if slices.output_artifact:
-                self.output_artifact_slices = {}
-                for name in slices.output_artifact:
-                    self.output_artifact_slices[name] = slices.slices
-                    self.outputs.artifacts[name].archive = None  # no archive
-            if slices.output_parameter:
-                for name in slices.output_parameter:
-                    self.output_parameter_slices[name] = slices.slices
+            self.add_slices(slices, input_artifact_prefix)
+        else:
+            self.render_script()
 
-            if slices.sub_path:
-                for name in slices.input_artifact:
-                    self.inputs.parameters["dflow_%s_sub_path" %
-                                           name] = InputParameter(value=".")
-                    sign = self.input_sign[name]
-                    self.inputs.artifacts[name] = InputArtifact(
-                        path="%s/inputs/artifacts/%s/{{inputs.parameters."
-                        "dflow_%s_sub_path}}" % (self.tmp_root, name, name),
-                        optional=sign.optional, type=sign.type)
+    def add_slices(self, slices: Slices, input_artifact_prefix=None):
+        if input_artifact_prefix is None:
+            input_artifact_prefix = {}
+        if slices.input_artifact and not slices.sub_path:
+            for name in slices.input_artifact:
+                if name in input_artifact_prefix:
+                    self.input_artifact_slices[name] = add_prefix_to_slice(
+                        input_artifact_prefix[name], slices.slices)
+                else:
+                    self.input_artifact_slices[name] = slices.slices
+        if slices.input_parameter:
+            for name in slices.input_parameter:
+                self.input_parameter_slices[name] = slices.slices
+        if slices.output_artifact:
+            self.output_artifact_slices = {}
+            for name in slices.output_artifact:
+                self.output_artifact_slices[name] = slices.slices
+                self.outputs.artifacts[name].archive = None  # no archive
+        if slices.output_parameter:
+            for name in slices.output_parameter:
+                self.output_parameter_slices[name] = slices.slices
+
+        if slices.sub_path:
+            for name in slices.input_artifact:
+                self.inputs.parameters["dflow_%s_sub_path" %
+                                       name] = InputParameter(value=".")
+                sign = self.input_sign[name]
+                self.inputs.artifacts[name] = InputArtifact(
+                    path="%s/inputs/artifacts/%s/{{inputs.parameters."
+                    "dflow_%s_sub_path}}" % (self.tmp_root, name, name),
+                    optional=sign.optional, type=sign.type)
+        self.render_script()
 
     def render_script(self):
         op_class = self.op_class
@@ -372,18 +397,8 @@ class PythonOPTemplate(PythonScriptOPTemplate):
 
         script = self.pre_script.format(**{"tmp_root": self.tmp_root})
         if self.python_packages:
-            script += "import os, sys, json\n"
-            script += "package_root = r'%s/inputs/artifacts/"\
-                "dflow_python_packages'\n" % self.tmp_root
-            script += "catalog_dir = os.path.join(package_root, "\
-                "'%s')\n" % config['catalog_dir_name']
-            script += "if os.path.exists(catalog_dir):\n"
-            script += "    for f in os.listdir(catalog_dir):\n"
-            script += "        with open(os.path.join(catalog_dir, f), 'r')"\
-                " as fd:\n"
-            script += "            for item in json.load(fd)['path_list']:\n"
-            script += "                sys.path.insert(0, os.path.join("\
-                "package_root, os.path.dirname(item['dflow_list_item'])))\n"
+            script += handle_packages_script(
+                "%s/inputs/artifacts/dflow_python_packages" % self.tmp_root)
 
         script += "import json, jsonpickle\n"
         script += "from dflow import config, s3_config\n"
@@ -398,6 +413,8 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                 else:
                     script += get_source_code(op_class)
             except Exception:
+                logging.info("Failed to get source code of OP, "
+                             "use cloudpickle instead", exc_info=True)
                 import cloudpickle
                 if self.python_packages:
                     self.python_packages.update(cloudpickle.__path__)
@@ -451,8 +468,16 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                     elif input_sign[name].type == Path:
                         script += "    input_sign['%s'].type = List[Path]\n" %\
                             name
+        if any([art.save_as_parameter
+                for art in self.inputs.artifacts.values()]):
+            script += "    pids = []\n"
         for name, sign in input_sign.items():
             if isinstance(sign, Artifact):
+                if self.inputs.artifacts[name].save_as_parameter:
+                    script += "    pids.append(jsonpickle.loads('{{inputs."\
+                        "parameters.dflow_art_%s}}').%s('%s', '%s/inputs/"\
+                        "artifacts/%s'))\n" % (name, self.download_method,
+                                               name, self.tmp_root, name)
                 slices = self.get_slices(input_artifact_slices, name)
                 if self.slices is not None and self.slices.sub_path and \
                         name in self.slices.input_artifact:
@@ -591,6 +616,13 @@ class PythonOPTemplate(PythonScriptOPTemplate):
             script += "        sys.exit(2)\n"
 
         script += self.post_script.format(**{"tmp_root": self.tmp_root})
+
+        if any([art.save_as_parameter
+                for art in self.inputs.artifacts.values()]):
+            script += "    import signal\n"
+            script += "    [os.killpg(os.getpgid(pid), signal.SIGTERM)"\
+                " for pid in pids if pid is not None]\n"
+
         self.script = script
 
     def get_slices(self, slices_dict, name):
@@ -617,6 +649,16 @@ class PythonOPTemplate(PythonScriptOPTemplate):
             i = slices.find("{{item")
         return slices
 
+    def convert_to_argo(self, memoize_prefix=None, memoize_configmap="dflow"):
+        if self.retry_on_transient_error is not None:
+            if self.timeout_as_transient_error:
+                expr = "asInt(lastRetry.exitCode) != 2"
+            else:
+                expr = "asInt(lastRetry.exitCode) == 1"
+            self.retry_strategy = V1alpha1RetryStrategy(
+                limit=self.retry_on_transient_error, expression=expr)
+        return super().convert_to_argo(memoize_prefix, memoize_configmap)
+
 
 class TransientError(Exception):
     pass
