@@ -1,12 +1,11 @@
-from typing import Optional, List
+from typing import List, Optional, Union
 
 from .io import OutputArtifact, OutputParameter
 from .op_template import OPTemplate
 from .step import Step
 
 try:
-    from argo.workflows.client import (V1alpha1Arguments, V1alpha1ContinueOn,
-                                       V1alpha1DAGTask)
+    from argo.workflows.client import V1alpha1Arguments, V1alpha1DAGTask
 except Exception:
     pass
 
@@ -42,7 +41,7 @@ class Task(Step):
             self,
             name: str,
             template: OPTemplate,
-            dependencies: Optional[List["Task"]] = None,
+            dependencies: Optional[List[Union["Task", str]]] = None,
             **kwargs,
     ) -> None:
         if dependencies is None:
@@ -86,6 +85,12 @@ class Task(Step):
 
     def convert_to_argo(self, context=None):
         self.prepare_argo_arguments(context)
+        depends = []
+        for task in self.dependencies:
+            if isinstance(task, Task):
+                depends.append("(%s.Succeeded)" % task)
+            else:
+                depends.append("(%s)" % task)
         return V1alpha1DAGTask(
             name=self.name, template=self.template.name,
             arguments=V1alpha1Arguments(
@@ -93,6 +98,5 @@ class Task(Step):
                 artifacts=self.argo_artifacts
             ), when=self.when, with_param=self.with_param,
             with_sequence=self.with_sequence,
-            continue_on=V1alpha1ContinueOn(failed=self.continue_on_failed),
-            dependencies=[task.name for task in self.dependencies],
+            depends=" && ".join(depends),
         )
