@@ -327,7 +327,8 @@ def download_s3(
     else:
         client = MinioClient(**kwargs)
     if recursive:
-        for obj in client.list(prefix=key, recursive=True):
+        from tqdm import tqdm
+        for obj in tqdm(client.list(prefix=key, recursive=True)):
             rel_path = obj[len(key):]
             if rel_path[:1] == "/":
                 rel_path = rel_path[1:]
@@ -778,3 +779,9 @@ class MinioClient(StorageClient):
     def get_md5(self, key: str) -> str:
         return self.client.stat_object(bucket_name=self.bucket_name,
                                        object_name=key).etag
+
+
+def add_prefix_to_slice(prefix, slices):
+    return "(lambda x: ['%s.' + str(i) for i in x] "\
+        "if isinstance(x, list) else '%s.' + str(x))(%s)" % (
+            prefix, prefix, slices)
