@@ -1,6 +1,7 @@
 import json
 import logging
 import os
+import sys
 import time
 from copy import deepcopy
 from typing import Any, Dict, List, Optional, Union
@@ -107,8 +108,8 @@ class Workflow:
             labels: Dict[str, str] = None,
             parallelism: Optional[int] = None,
             pod_gc_strategy: Optional[str] = None,
-            image_pull_secrets: Union[str, DockerSecret,
-                                      List[Union[str, DockerSecret]]] = None,
+            image_pull_secrets: Optional[Union[
+                str, DockerSecret, List[Union[str, DockerSecret]]]] = None,
             artifact_repo_key: Optional[str] = None,
             parameters: Optional[Dict[str, Any]] = None,
     ) -> None:
@@ -158,23 +159,9 @@ class Workflow:
             parameters = {}
         self.parameters = parameters
         self.k8s_client = None
-
-        if self.artifact_repo_key is not None:
-            core_v1_api = self.get_k8s_core_v1_api()
-            cm = core_v1_api.read_namespaced_config_map(
-                namespace=self.namespace, name="artifact-repositories")
-            repo = yaml.full_load(cm.data[self.artifact_repo_key])
-            s3_config["repo"] = repo
-            if "s3" in repo:
-                s3_config["repo_type"] = "s3"
-                s3 = repo["s3"]
-            elif "oss" in repo:
-                s3_config["repo_type"] = "oss"
-                s3 = repo["oss"]
-            if "keyFormat" in s3:
-                t = "{{workflow.name}}/{{pod.name}}"
-                if s3["keyFormat"].endswith(t):
-                    s3_config["repo_prefix"] = s3["keyFormat"][:-len(t)]
+        parse_repo(self.artifact_repo_key, self.namespace,
+                   k8s_api_server=self.k8s_api_server, token=self.token,
+                   k8s_config_file=self.k8s_config_file)
 
     def get_k8s_core_v1_api(self):
         if self.k8s_client is None:
@@ -225,6 +212,9 @@ class Workflow:
                     if not os.path.exists(wfdir):
                         os.makedirs(wfdir)
                         break
+            else:
+                wfdir = os.path.abspath(self.id)
+                os.makedirs(wfdir, exist_ok=True)
 
             if reuse_step is not None:
                 for step in reuse_step:
@@ -292,6 +282,14 @@ class Workflow:
             print("Workflow is running locally (ID: %s)" % self.id)
             with open(os.path.join(wfdir, "status"), "w") as f:
                 f.write("Running")
+            if config["detach"]:
+                pid = os.fork()
+                if pid != 0:
+                    os.chdir(cwd)
+                    return ArgoWorkflow({"id": self.id})
+                flog = open(os.path.join(wfdir, "log.txt"), "w")
+                os.dup2(flog.fileno(), sys.stdout.fileno())
+                os.dup2(flog.fileno(), sys.stderr.fileno())
             try:
                 entrypoint = deepcopy(self.entrypoint)
                 entrypoint.orig_template = self.entrypoint
@@ -303,10 +301,12 @@ class Workflow:
                 traceback.print_exc()
                 with open(os.path.join(wfdir, "status"), "w") as f:
                     f.write("Failed")
+            if config["detach"]:
+                flog.close()
+                exit()
             os.chdir(cwd)
             return ArgoWorkflow({"id": self.id})
 
-        assert self.id is None, "Do not submit a workflow repeatedly"
         manifest = self.convert_to_argo(reuse_step=reuse_step)
 
         logger.debug("submit manifest:\n%s" % manifest)
@@ -427,7 +427,8 @@ class Workflow:
         if reuse_step is not None:
             self.reused_keys = [step.key for step in reuse_step
                                 if step.key is not None]
-            self.id = self.name + "-" + randstr()
+            if self.id is None:
+                self.id = self.name + "-" + randstr()
             self.copied_keys = []
             self.memoize_map = {}
             key2id = {}
@@ -605,7 +606,7 @@ class Workflow:
         if d.get("metadata", {}).get("annotations", {}).get(
                 "workflow.dp.tech/executor") == "dispatcher":
             host = kwargs["annotations"].get("workflow.dp.tech/host")
-            port = kwargs["annotations"].get("workflow.dp.tech/port", 22)
+            port = int(kwargs["annotations"].get("workflow.dp.tech/port", 22))
             username = kwargs["annotations"].get(
                 "workflow.dp.tech/username", "root")
             password = kwargs["annotations"].get("workflow.dp.tech/password")
@@ -616,12 +617,14 @@ class Workflow:
             machine = extras.get("machine", None)
             resources = extras.get("resources", None)
             task = extras.get("task", None)
+            clean = extras.get("clean", True)
             from .plugins.dispatcher import DispatcherExecutor
             kwargs["context"] = DispatcherExecutor(
                 host, queue_name, port, username, password,
                 machine_dict=machine, resources_dict=resources,
                 task_dict=task, docker_executable=docker,
-                singularity_executable=singularity, podman_executable=podman)
+                singularity_executable=singularity, podman_executable=podman,
+                clean=clean)
         elif engine:
             from .executor import ContainerExecutor
             kwargs["context"] = ContainerExecutor(docker, singularity, podman)
@@ -696,7 +699,7 @@ class Workflow:
         if config["mode"] == "debug":
             with open("%s/status" % self.id, "r") as f:
                 return f.read()
-        workflow = self.query(fields=["status"])
+        workflow = self.query(fields=["status.phase"])
 
         if "phase" not in workflow.status:
             return "Pending"
@@ -738,7 +741,7 @@ class Workflow:
                 stepdir = os.path.join(self.id, s)
                 if not os.path.isdir(stepdir):
                     continue
-                if name is not None and not s[:len(name)] == name:
+                if name is not None and not s.startswith(self.id + "-" + name):
                     continue
                 if key is not None and s not in key:
                     continue
@@ -1027,3 +1030,27 @@ def query_archived_workflows(
             query_params=query_params,
             _return_http_data_only=True)
     return [ArgoWorkflow(w) for w in res["items"]] if res["items"] else []
+
+
+def parse_repo(repo_key=None, namespace=None, **kwargs):
+    if repo_key is None:
+        repo_key = s3_config["repo_key"]
+    if namespace is None:
+        namespace = config["namespace"]
+    if repo_key is not None and s3_config["repo"] is None:
+        with get_k8s_client(**kwargs) as k8s_client:
+            core_v1_api = kubernetes.client.CoreV1Api(k8s_client)
+            cm = core_v1_api.read_namespaced_config_map(
+                namespace=namespace, name="artifact-repositories")
+            repo = yaml.full_load(cm.data[repo_key])
+            s3_config["repo"] = repo
+            if "s3" in repo:
+                s3_config["repo_type"] = "s3"
+                s3 = repo["s3"]
+            elif "oss" in repo:
+                s3_config["repo_type"] = "oss"
+                s3 = repo["oss"]
+            if "keyFormat" in s3:
+                t = "{{workflow.name}}/{{pod.name}}"
+                if s3["keyFormat"].endswith(t):
+                    s3_config["repo_prefix"] = s3["keyFormat"][:-len(t)]
